@@ -17,6 +17,7 @@ cleanup=YES
 preprocess=0
 manpages=0
 qhppages=0
+api_searchbox=0
 searchengine=0
 
 while test -n "$1" ; do
@@ -47,6 +48,9 @@ case $1 in
 --qhppages)
         qhppages=1
         ;;
+--api-searchbox)
+	api_searchbox=1
+	;;
 --searchengine)
         searchengine=1
         ;;
@@ -61,6 +65,7 @@ case $1 in
 	echo "  --preprocess     Generate source code (KConfigXT, uic, etc.)"
         echo "  --manpages       Generate man pages in addition to html"
         echo "  --qhppages       Generate pages for Qt Assistant"
+	echo "  --api-searchbox  Add an api search box in all pages, like api.kde.org's"
         echo "  --searchengine   Enable search engine"
 	exit 2
 	;;
@@ -205,7 +210,7 @@ fi
 #
 preprocess_sources()
 {
-	for dir in `find $top_srcdir/$subdir -type d | grep -v ".svn"`; do
+	for dir in `find $top_srcdir/$subdir -type d | grep -v ".svn" | grep -v ".git"`; do
 		# execute global preprocessing modules
 		local preproc_base="`dirname $0`"
 		for proc in `find "$preproc_base" -name "doxygen-preprocess-*.sh"`; do
@@ -228,7 +233,7 @@ fi
 ### We need some values from top-level files, which
 ### are not preserved between invocations of this
 ### script, so factor it out for easy use.
-create_doxyfile_in() 
+create_doxyfile_in()
 {
 	VERSION=`grep '^/.*DOXYGEN_VERSION' "$top_srcdir/Mainpage.dox" | sed -e 's+.*=++'`
 	NAME=`grep '^/.*DOXYGEN_NAME' "$top_srcdir/Mainpage.dox" | sed -e 's+.*=++'`
@@ -246,7 +251,7 @@ if test -z "$subdir" ; then
 	if test ! -d "$apidoxdir" ; then
 		mkdir "$apidoxdir" > /dev/null 2>&1
 	fi
-	cd "$apidoxdir" > /dev/null 2>&1 || { 
+	cd "$apidoxdir" > /dev/null 2>&1 || {
 		echo "Cannot create and cd into $apidoxdir"
 		exit 1
 	}
@@ -292,7 +297,8 @@ extract_line()
 	test -f "$file" || return
 	pattern=`echo "$1" | tr + .`
 	grep "^//[[:space:]]*$1" "$file" | \
-		sed -e "s+//[[:space:]]*$pattern.*=[[:space:]]*++"
+        sed -e "s+//[[:space:]]*$pattern.*=[[:space:]]*++" | \
+        sed -e "s+@currentdir@+$subdir+g"
 }
 
 ### Try to order the top-level subdirectories so that we get the smallest
@@ -301,7 +307,7 @@ create_subdirs()
 {
 echo "* Sorting top-level subdirs"
 
-( cd "$top_srcdir" && 
+( cd "$top_srcdir" &&
 for dir in *
 do
 	# For each Mainpage.dox, get the list (possibly empty) of
@@ -348,7 +354,7 @@ apidox_qhppages()
                PROJECT_NAME="KDE"
         fi
         if test -z "$PROJECT_VERSION" ; then
-               PROJECT_VERSION="4.4"
+               PROJECT_VERSION="4.8"
         fi
         vf="$PROJECT_NAME"-"$PROJECT_VERSION"
         echo "QHP_VIRTUAL_FOLDER     = $vf" >> "$subdir/Doxyfile"
@@ -405,6 +411,7 @@ apidox_local()
 # In non-top directories, both <!-- menu --> and <!-- gmenu -->
 # are calculated and replaced. Top directories get an empty <!-- menu -->
 # if any.
+# <!-- api_searchbox --> gets replaced by a search box like api.kde.org's.
 doxyndex()
 {
 	# Special case top-level to have an empty MENU.
@@ -437,15 +444,16 @@ doxyndex()
 		# into parts. For each, if the file part exists (as a html
 		# file) tack it onto the MENU variable as a <li> with link.
 		for i in "Main Page/index" \
-			"Modules/modules" \
 			"Namespace List/namespaces" \
-			"Class Hierarchy/hierarchy" \
+			"Namespace Members/namespacemembers" \
 			"Alphabetical List/classes" \
 			"Class List/annotated" \
+			"Class Hierarchy/hierarchy" \
+                        "Class Members/functions" \
 			"File List/files" \
+                        "File Members/globals" \
+			"Modules/modules" \
 			"Directories/dirs" \
-			"Namespace Members/namespacemembers" \
-			"Class Members/functions" \
 			"Related Pages/pages"
 		do
 			NAME=`dirname "$i"`
@@ -499,6 +507,16 @@ doxyndex()
 				-e "s+<!-- cmenu.end -->+$CMENUEND+" \
 				< "$i"  | sed -e "s+@topdir@+$htmltop+g" | sed -e "s+@topname@+$TOPNAME+g" | sed -e "s+@copyright@+$COPYRIGHT+g" > "$i.new" && mv "$i.new" "$i"
 			sed -e "s+<!-- cmenu -->+$CMENU+" < "$i" > "$i.new"
+
+		        # API_SEARCHBOX substitution.
+		        # It adds the multi-module search box found in api.kde.org's main page.
+		        # FIXME: it currently only makes sense to enable it in api.kde.org's
+		        #        context, as it hardcodes the list of modules and requires AKO's
+		        #        mapsearcher.php to work.
+			if test "${api_searchbox}" = "1"; then
+			    sed -i -e "/<!-- api_searchbox -->/r ${DOXDATA}/api_searchbox.html" ${i}.new
+			fi
+
 			test -s "$i.new" && mv "$i.new" "$i"
 			echo "* Added menus to $i"
 		fi
@@ -553,7 +571,7 @@ apidox_toplevel()
 		echo "OUTPUT_DIRECTORY       = $top_builddir"
 		echo "RECURSIVE              = NO"
 		echo "ALPHABETICAL_INDEX     = NO"
-		echo "HTML_OUTPUT            = ." 
+		echo "HTML_OUTPUT            = ."
 	} >> Doxyfile
 	apidox_htmlfiles "main"
         if test "$manpages" = "1"; then
@@ -613,7 +631,7 @@ apidox_toplevel()
 # $1 is empty to generate tag files only.
 # $1 is non-empty to generate HTML.
 #
-# Additionally, the environment variable $NO_APPEND_TAG may be set 
+# Additionally, the environment variable $NO_APPEND_TAG may be set
 # to a non-empty value to suppress adding the tag for this subdir
 # to the list of tag files (used when re-generating a single subdir).
 #
