@@ -257,8 +257,8 @@ void KTextEdit::Private::init()
 {
     spellInterface = 0;
     KCursor::setAutoHideCursor(parent, true, false);
-    parent->connect(parent, SIGNAL(languageChanged(const QString&)),
-                    parent, SLOT(setSpellCheckingLanguage(const QString&)));
+    parent->connect(parent, SIGNAL(languageChanged(QString)),
+                    parent, SLOT(setSpellCheckingLanguage(QString)));
 }
 
 KTextEdit::KTextEdit( const QString& text, QWidget *parent )
@@ -310,11 +310,11 @@ void KTextEdit::showSpellConfigDialog(const QString &configFileName,
     Sonnet::ConfigDialog dialog(&config, this);
     if (!d->spellCheckingLanguage.isEmpty())
         dialog.setLanguage(d->spellCheckingLanguage);
-    connect(&dialog, SIGNAL(languageChanged(const QString &)),
-            this, SLOT(setSpellCheckingLanguage(const QString &)));
     if (!windowIcon.isEmpty())
         dialog.setWindowIcon(KIcon(windowIcon));
-    dialog.exec();
+    if(dialog.exec()) {
+        setSpellCheckingLanguage( dialog.language() );
+    }
 }
 
 bool KTextEdit::event(QEvent* ev)
@@ -422,16 +422,14 @@ bool KTextEdit::Private::handleShortcut(const QKeyEvent* event)
     cursor.movePosition( QTextCursor::EndOfLine );
     parent->setTextCursor( cursor );
     return true;
-  } else if (KStandardShortcut::find().contains(key)) {
-      if (findReplaceEnabled)
-          parent->slotFind();
+  } else if (findReplaceEnabled && KStandardShortcut::find().contains(key)) {
+      parent->slotFind();
       return true;
-  } else if (KStandardShortcut::findNext().contains(key)) {
-      if (findReplaceEnabled)
-          parent->slotFindNext();
+  } else if (findReplaceEnabled && KStandardShortcut::findNext().contains(key)) {
+      parent->slotFindNext();
       return true;
-  } else if (KStandardShortcut::replace().contains(key)) {
-      if (!parent->isReadOnly() && findReplaceEnabled)
+  } else if (findReplaceEnabled && KStandardShortcut::replace().contains(key)) {
+      if (!parent->isReadOnly())
           parent->slotReplace();
       return true;
   } else if ( KStandardShortcut::pasteSelection().contains( key ) ) {
@@ -464,8 +462,8 @@ QMenu *KTextEdit::mousePopupMenu()
 {
   QMenu *popup = createStandardContextMenu();
   if (!popup) return 0;
-  connect( popup, SIGNAL( triggered ( QAction* ) ),
-             this, SLOT( menuActivated( QAction* ) ) );
+  connect( popup, SIGNAL(triggered(QAction*)),
+             this, SLOT(menuActivated(QAction*)) );
 
   const bool emptyDocument = document()->isEmpty();
   if( !isReadOnly() )
@@ -502,23 +500,26 @@ QMenu *KTextEdit::mousePopupMenu()
       d->allowTab = popup->addAction( i18n("Allow Tabulations") );
       d->allowTab->setCheckable( true );
       d->allowTab->setChecked( !tabChangesFocus() );
+  }
 
-      if (d->findReplaceEnabled)
-      {
-          KAction *findAction = KStandardAction::find(this, SLOT(slotFind()), popup);
-          KAction *findNextAction = KStandardAction::findNext(this, SLOT(slotFindNext()), popup);
+  if (d->findReplaceEnabled) {
+      KAction *findAction = KStandardAction::find(this, SLOT(slotFind()), popup);
+      KAction *findNextAction = KStandardAction::findNext(this, SLOT(slotFindNext()), popup);
+      if (emptyDocument) {
+          findAction->setEnabled(false);
+          findNextAction->setEnabled(false);
+      } else {
+          findNextAction->setEnabled(d->find != 0);
+      }
+      popup->addSeparator();
+      popup->addAction(findAction);
+      popup->addAction(findNextAction);
+
+      if (!isReadOnly()) {
           KAction *replaceAction = KStandardAction::replace(this, SLOT(slotReplace()), popup);
-          if (emptyDocument)
-          {
-              findAction->setEnabled(false);
-              findNextAction->setEnabled(false );
+          if (emptyDocument) {
               replaceAction->setEnabled(false);
           }
-	  else
-	      findNextAction->setEnabled(d->find != 0 );
-          popup->addSeparator();
-          popup->addAction(findAction);
-          popup->addAction(findNextAction);
           popup->addAction(replaceAction);
       }
   }
@@ -792,22 +793,22 @@ void KTextEdit::checkSpelling()
      backgroundSpellCheck->changeLanguage(d->spellCheckingLanguage);
   Sonnet::Dialog *spellDialog = new Sonnet::Dialog(
       backgroundSpellCheck, 0);
-  connect(spellDialog, SIGNAL(replace( const QString&, int,const QString&)),
-          this, SLOT(spellCheckerCorrected( const QString&, int,const QString&)));
-  connect(spellDialog, SIGNAL(misspelling( const QString&, int)),
-          this, SLOT(spellCheckerMisspelling(const QString &,int)));
-  connect(spellDialog, SIGNAL(autoCorrect(const QString&, const QString&)),
-          this, SLOT(spellCheckerAutoCorrect(const QString&, const QString&)));
-  connect(spellDialog, SIGNAL(done(const QString&)),
+  connect(spellDialog, SIGNAL(replace(QString,int,QString)),
+          this, SLOT(spellCheckerCorrected(QString,int,QString)));
+  connect(spellDialog, SIGNAL(misspelling(QString,int)),
+          this, SLOT(spellCheckerMisspelling(QString,int)));
+  connect(spellDialog, SIGNAL(autoCorrect(QString,QString)),
+          this, SLOT(spellCheckerAutoCorrect(QString,QString)));
+  connect(spellDialog, SIGNAL(done(QString)),
           this, SLOT(spellCheckerFinished()));
   connect(spellDialog, SIGNAL(cancel()),
           this, SLOT(spellCheckerCanceled()));
   connect(spellDialog, SIGNAL(stop()),
           this, SLOT(spellCheckerFinished()));
-  connect(spellDialog, SIGNAL(spellCheckStatus(const QString &)),
-          this,SIGNAL(spellCheckStatus(const QString &)));
-  connect(spellDialog, SIGNAL(languageChanged(const QString &)),
-          this, SIGNAL(languageChanged(const QString &)));
+  connect(spellDialog, SIGNAL(spellCheckStatus(QString)),
+          this,SIGNAL(spellCheckStatus(QString)));
+  connect(spellDialog, SIGNAL(languageChanged(QString)),
+          this, SIGNAL(languageChanged(QString)));
   d->originalDoc = QTextDocumentFragment(document());
   spellDialog->setBuffer(toPlainText());
   spellDialog->show();
@@ -860,11 +861,11 @@ void KTextEdit::slotDoReplace()
 
     // Connect highlight signal to code which handles highlighting
     // of found text.
-    connect(d->replace, SIGNAL(highlight(const QString &, int, int)),
-            this, SLOT(slotFindHighlight(const QString &, int, int)));
+    connect(d->replace, SIGNAL(highlight(QString,int,int)),
+            this, SLOT(slotFindHighlight(QString,int,int)));
     connect(d->replace, SIGNAL(findNext()), this, SLOT(slotReplaceNext()));
-    connect(d->replace, SIGNAL(replace(const QString &, int, int, int)),
-            this, SLOT(slotReplaceText(const QString &, int, int, int)));
+    connect(d->replace, SIGNAL(replace(QString,int,int,int)),
+            this, SLOT(slotReplaceText(QString,int,int,int)));
 
     d->repDlg->close();
     slotReplaceNext();
@@ -934,8 +935,8 @@ void KTextEdit::slotDoFind()
 
     // Connect highlight signal to code which handles highlighting
     // of found text.
-    connect(d->find, SIGNAL(highlight(const QString &, int, int)),
-            this, SLOT(slotFindHighlight(const QString &, int, int)));
+    connect(d->find, SIGNAL(highlight(QString,int,int)),
+            this, SLOT(slotFindHighlight(QString,int,int)));
     connect(d->find, SIGNAL(findNext()), this, SLOT(slotFindNext()));
 
     d->findDlg->close();
@@ -1049,11 +1050,11 @@ bool KTextEdit::Private::overrideShortcut(const QKeyEvent* event)
     return true;
   } else if ( KStandardShortcut::pasteSelection().contains( key ) ) {
     return true;
-  } else if (KStandardShortcut::find().contains(key)) {
+  } else if (findReplaceEnabled && KStandardShortcut::find().contains(key)) {
       return true;
-  } else if (KStandardShortcut::findNext().contains(key)) {
+  } else if (findReplaceEnabled && KStandardShortcut::findNext().contains(key)) {
       return true;
-  } else if (KStandardShortcut::replace().contains(key)) {
+  } else if (findReplaceEnabled && KStandardShortcut::replace().contains(key)) {
       return true;
   } else if (event->matches(QKeySequence::SelectAll)) { // currently missing in QTextEdit
       return true;
