@@ -25,7 +25,6 @@
 
 #include "khtml_part.h"
 #include "dom/dom_string.h"
-#include "misc/htmlhashes.h"
 #include "imload/imagemanager.h"
 #include "khtmlview.h"
 #include <QtCore/QCharRef>
@@ -176,12 +175,12 @@ void HTMLObjectBaseElementImpl::parseAttribute(AttributeImpl *attr)
     {
         case ATTR_TYPE:
         case ATTR_CODETYPE:
-	    if (attr->val()) {
-	      DOM::DOMStringImpl *stringImpl = attr->val();
-	      QString val = QString::fromRawData( stringImpl->s, stringImpl->l );
-	      setServiceType( val );
-              setNeedComputeContent();
-	    }
+            if (attr->val()) {
+                DOM::DOMStringImpl *stringImpl = attr->val();
+                QString val = QString( stringImpl->s, stringImpl->l );
+                setServiceType( val );
+                setNeedComputeContent();
+            }
             break;
         case ATTR_WIDTH:
             if (!attr->value().isEmpty())
@@ -271,7 +270,7 @@ void HTMLObjectBaseElementImpl::requestRerender()
 {
     if (m_rerender) return;
     m_rerender = true;
-    QTimer::singleShot( 0, this, SLOT( slotRerender()) );
+    QTimer::singleShot( 0, this, SLOT(slotRerender()) );
 }
 
 void HTMLObjectBaseElementImpl::slotRerender()
@@ -510,6 +509,12 @@ void HTMLObjectBaseElementImpl::computeContent()
         KHTMLPart* p = document()->part();
         if (!p || !p->javaEnabled())
             newRenderAlternative = true;
+    } else {    
+        // Similarly for plugins.
+        KHTMLPart* p = document()->part();
+
+        if (!p || !p->pluginsEnabled())
+            newRenderAlternative = true;
     }
 
     // If there is no <embed> (here or as a child), and we don't have a type + url to go on,
@@ -530,7 +535,7 @@ void HTMLObjectBaseElementImpl::computeContent()
 
     kDebug(6031) << effectiveURL << effectiveServiceType << params;
 
-    if (!part->requestObject( this, effectiveURL, effectiveServiceType, params)) {
+    if (!part->loadObjectElement( this, effectiveURL, effectiveServiceType, params)) {
         // Looks like we are gonna need alternative content after all...
         m_renderAlternative = true;
     }
@@ -741,11 +746,6 @@ void HTMLEmbedElementImpl::parseAttribute(AttributeImpl *attr)
 
 void HTMLEmbedElementImpl::attach()
 {
-    KHTMLPart* p = document()->part();
-
-    if (!p || !p->pluginsEnabled())
-        m_renderAlternative = true;
-
     if (parentNode()->id() == ID_OBJECT)
         NodeBaseImpl::attach();
     else
@@ -828,11 +828,6 @@ DocumentImpl* HTMLObjectElementImpl::contentDocument() const
 
 void HTMLObjectElementImpl::attach()
 {
-    KHTMLPart* p = document()->part();
-
-    if (!p || !p->pluginsEnabled())
-        m_renderAlternative = true;
-
     HTMLObjectBaseElementImpl::attach();
 }
 
@@ -862,3 +857,4 @@ void HTMLParamElementImpl::parseAttribute(AttributeImpl *attr)
 }
 
 #include "html_objectimpl.moc"
+// kate: indent-width 4; replace-tabs on; tab-width 4; space-indent on;
