@@ -38,7 +38,6 @@
 #include "dtd.h"
 
 #include <misc/loader.h>
-#include <misc/htmlhashes.h>
 
 #include <khtmlview.h>
 #include <khtml_part.h>
@@ -141,17 +140,18 @@ HTMLTokenizer::HTMLTokenizer(DOM::DocumentImpl *_doc, KHTMLView *_view)
 {
     view = _view;
     buffer = 0;
-    scriptCode = 0;
-    scriptCodeSize = scriptCodeMaxSize = scriptCodeResync = 0;
+    rawContent = 0;
+    rawContentSize = rawContentMaxSize = rawContentResync = rawContentSinceLastEntity = 0;
     charsets = KGlobal::charsets();
     parser = new KHTMLParser(_view, _doc);
     m_executingScript = 0;
-    m_autoCloseTimer = 0;
+    m_externalScriptsTimerId = 0;
     m_tokenizerYieldDelay = sTokenizerFastYieldDelay;
     m_yieldTimer = 0;
     m_prospectiveTokenizer = 0;
     onHold = false;
     m_documentTokenizer = true;
+    m_hasScriptsWaitingForStylesheets = false;
 
     reset();
 }
@@ -160,17 +160,18 @@ HTMLTokenizer::HTMLTokenizer(DOM::DocumentImpl *_doc, DOM::DocumentFragmentImpl
 {
     view = 0;
     buffer = 0;
-    scriptCode = 0;
-    scriptCodeSize = scriptCodeMaxSize = scriptCodeResync = 0;
+    rawContent = 0;
+    rawContentSize = rawContentMaxSize = rawContentResync = rawContentSinceLastEntity = 0;
     charsets = KGlobal::charsets();
     parser = new KHTMLParser( i, _doc );
     m_executingScript = 0;
-    m_autoCloseTimer = 0;
+    m_externalScriptsTimerId = 0;
     m_tokenizerYieldDelay = sTokenizerFastYieldDelay;
     m_yieldTimer = 0;
     m_prospectiveTokenizer = 0;
     onHold = false;
     m_documentTokenizer = false;
+    m_hasScriptsWaitingForStylesheets = false;
 
     reset();
 }
@@ -194,22 +195,23 @@ void HTMLTokenizer::reset()
     buffer = dest = 0;
     size = 0;
 
-    if ( scriptCode )
-        KHTML_DELETE_QCHAR_VEC(scriptCode);
-    scriptCode = 0;
-    scriptCodeSize = scriptCodeMaxSize = scriptCodeResync = 0;
-
-    if (m_autoCloseTimer > 0) {
-        killTimer(m_autoCloseTimer);
-        m_autoCloseTimer = 0;
-    }
+    if ( rawContent )
+        KHTML_DELETE_QCHAR_VEC(rawContent);
+    rawContent = 0;
+    rawContentSize = rawContentMaxSize = rawContentResync = 0;
 
     if (m_yieldTimer > 0) {
         killTimer(m_yieldTimer);
         m_yieldTimer = 0;
     }
+
+    if (m_externalScriptsTimerId > 0) {
+        killTimer(m_externalScriptsTimerId);
+        m_externalScriptsTimerId = 0;
+    }
     currToken.reset();
     doctypeToken.reset();
+    javascript = false;
 }
 
 void HTMLTokenizer::begin()
@@ -249,7 +251,6 @@ void HTMLTokenizer::begin()
     noMoreData = false;
     brokenComments = false;
     brokenServer = false;
-    brokenScript = false;
     lineno = 0;
     scriptStartLineno = 0;
     tagStartLineno = 0;
@@ -340,8 +341,15 @@ void HTMLTokenizer::processListing(TokenizerString list)
     pre = old_pre;
 }
 
-void HTMLTokenizer::parseSpecial(TokenizerString &src)
+void HTMLTokenizer::parseRawContent(TokenizerString &src)
 {
+    // The 'raw content' mode is a very lax tokenizing mode
+    // that will absorb anything but the exact closing tag
+    // that made us enter this mode, *except* if it inside a comment.
+    //
+    // Any other tag or comment will be passed verbatim to the parser as part
+    // of the content. It is used for script, style, and a few others.
+    // 
     assert( textarea || title || !Entity );
     assert( !tag );
     assert( xmp+textarea+title+style+script == 1 );
@@ -351,24 +359,26 @@ void HTMLTokenizer::parseSpecial(TokenizerString &src)
     if ( comment ) parseComment( src );
 
     while ( !src.isEmpty() ) {
-        checkScriptBuffer();
+        checkRawContentBuffer();
         unsigned char ch = src->toLatin1();
-        if ( !scriptCodeResync && !brokenComments && !title && !textarea && !xmp && ch == '-' && scriptCodeSize >= 3 && !src.escaped() && QString::fromRawData( scriptCode+scriptCodeSize-3, 3 ) == "<!-" ) {
+        if ( !rawContentResync && !brokenComments && !xmp && ch == '-' && 
+              rawContentSize >= 3 && ((!textarea && !title) || rawContentSinceLastEntity >= 3) && !src.escaped() && 
+              QString::fromRawData( rawContent+rawContentSize-3, 3 ) == "<!-" ) {
             comment = true;
-            scriptCode[ scriptCodeSize++ ] = ch;
+            rawContent[ rawContentSize++ ] = ch;
             ++src;
             parseComment( src );
             continue;
         }
-        if ( scriptCodeResync && !tquote && ( ch == '>' ) ) {
+        if ( rawContentResync && !tquote && ( ch == '>' ) ) {
             ++src;
-            scriptCodeSize = scriptCodeResync-1;
-            scriptCodeResync = 0;
-            scriptCode[ scriptCodeSize ] = scriptCode[ scriptCodeSize + 1 ] = 0;
+            rawContentSize = rawContentResync-1;
+            rawContentResync = 0;
+            rawContent[ rawContentSize ] = rawContent[ rawContentSize + 1 ] = 0;
             if ( script )
                 scriptHandler();
             else {
-                processListing(TokenizerString(scriptCode, scriptCodeSize));
+                processListing(TokenizerString(rawContent, rawContentSize));
                 processToken();
                 if ( style )         { currToken.tid = ID_STYLE + ID_CLOSE_TAG; }
                 else if ( textarea ) { currToken.tid = ID_TEXTAREA + ID_CLOSE_TAG; }
@@ -377,20 +387,25 @@ void HTMLTokenizer::parseSpecial(TokenizerString &src)
                 processToken();
                 script = style = textarea = title = xmp = false;
                 tquote = NoQuote;
-                scriptCodeSize = scriptCodeResync = 0;
+                rawContentSize = rawContentResync = 0;
             }
             return;
         }
         // possible end of tagname, lets check.
-        if ( !scriptCodeResync && !escaped && !src.escaped() && ( ch == '>' || ch == '/' || ch <= ' ' ) && ch &&
-             scriptCodeSize >= searchStopperLen &&
-             QString::compare(QString::fromRawData(scriptCode + scriptCodeSize - searchStopperLen, searchStopperLen),
-                 QLatin1String(searchStopper), Qt::CaseInsensitive) == 0) {
-            scriptCodeResync = scriptCodeSize-searchStopperLen+1;
+        if ( !rawContentResync && !escaped && !src.escaped() && ( ch == '>' || ch == '/' || ch <= ' ' ) && ch &&
+              rawContentSize >= searchStopperLen && ((!textarea && !title) || rawContentSinceLastEntity >= searchStopperLen) &&
+              QString::compare(QString::fromRawData(rawContent + rawContentSize - searchStopperLen, searchStopperLen),
+                  QLatin1String(searchStopper), Qt::CaseInsensitive) == 0) {
+
+            // the purpose of rawContentResync is to look for an end tag that could possibly be of the form:
+            // </endtag   junk="more junk>\"><>"     >
+            // IOW, once the '</endtag' sequence has been found, the rest of the tag must still be validated,
+            // so this micro-tokenizer switches to rawContentResync state until '>' is finally found.
+            rawContentResync = rawContentSize-searchStopperLen+1;
             tquote = NoQuote;
             continue;
         }
-        if ( scriptCodeResync && !escaped ) {
+        if ( rawContentResync && !escaped ) {
             if(ch == '\"')
                 tquote = (tquote == NoQuote) ? DoubleQuote : ((tquote == SingleQuote) ? SingleQuote : NoQuote);
             else if(ch == '\'')
@@ -399,15 +414,16 @@ void HTMLTokenizer::parseSpecial(TokenizerString &src)
                 tquote = NoQuote;
         }
         escaped = ( !escaped && ch == '\\' );
-        if (!scriptCodeResync && (textarea||title) && !src.escaped() && ch == '&') {
-            QChar *scriptCodeDest = scriptCode+scriptCodeSize;
+        if (!rawContentResync && (textarea||title) && !src.escaped() && ch == '&') {
+            QChar *rawContentDest = rawContent+rawContentSize;
             ++src;
-            parseEntity(src,scriptCodeDest,true);
-            scriptCodeSize = scriptCodeDest-scriptCode;
+            parseEntity(src,rawContentDest,true);
+            rawContentSize = rawContentDest-rawContent;
         }
         else {
-            scriptCode[ scriptCodeSize++ ] = *src;
+            rawContent[ rawContentSize++ ] = *src;
             ++src;
+            ++rawContentSinceLastEntity;
         }
     }
 }
@@ -417,7 +433,7 @@ void HTMLTokenizer::scriptHandler()
     QString currentScriptSrc = scriptSrc;
     scriptSrc.clear();
 
-    processListing(TokenizerString(scriptCode, scriptCodeSize));
+    processListing(TokenizerString(rawContent, rawContentSize));
     QString exScript( buffer, dest-buffer );
 
     processToken();
@@ -442,7 +458,7 @@ void HTMLTokenizer::scriptHandler()
             pendingQueue.push(src);
             int scriptCount = cachedScript.count();
             setSrc(TokenizerString());
-            scriptCodeSize = scriptCodeResync = 0;
+            rawContentSize = rawContentResync = 0;
             cs->ref(this);
             if (cachedScript.count() == scriptCount)
                 deferredScript = true;
@@ -450,7 +466,7 @@ void HTMLTokenizer::scriptHandler()
         else if (currentScriptSrc.isEmpty() && view && javascript ) {
             pendingQueue.push(src);
             setSrc(TokenizerString());
-            scriptCodeSize = scriptCodeResync = 0;
+            rawContentSize = rawContentResync = 0;
             scriptExecution( exScript, QString(), tagStartLineno /*scriptStartLineno*/ );
         } else {
             // script was filtered or disallowed
@@ -459,7 +475,7 @@ void HTMLTokenizer::scriptHandler()
     }
 
     script = false;
-    scriptCodeSize = scriptCodeResync = 0;
+    rawContentSize = rawContentResync = 0;
 
     if ( !effectiveScript )
         return;
@@ -505,9 +521,9 @@ void HTMLTokenizer::scriptExecution( const QString& str, const QString& scriptUR
 
 void HTMLTokenizer::parseComment(TokenizerString &src)
 {
-    checkScriptBuffer(src.length());
+    checkRawContentBuffer(src.length());
     while ( src.length() ) {
-        scriptCode[ scriptCodeSize++ ] = *src;
+        rawContent[ rawContentSize++ ] = *src;
 
 #if defined(TOKEN_DEBUG) && TOKEN_DEBUG > 1
         qDebug("comment is now: *%s*", src.toString().left(16).toLatin1().constData());
@@ -517,8 +533,8 @@ void HTMLTokenizer::parseComment(TokenizerString &src)
         {
             bool handleBrokenComments =  brokenComments && !( script || style );
             bool scriptEnd=false;
-            if ( scriptCodeSize > 2 && scriptCode[scriptCodeSize-3] == '-' &&
-                  scriptCode[scriptCodeSize-2] == '-' )
+            if ( rawContentSize > 2 && rawContent[rawContentSize-3] == '-' &&
+                  rawContent[rawContentSize-2] == '-' )
             {
                 scriptEnd=true;
             }
@@ -526,15 +542,16 @@ void HTMLTokenizer::parseComment(TokenizerString &src)
             if (handleBrokenComments || scriptEnd ){
                 ++src;
                 if ( !( title || script || xmp || textarea || style) ) {
-                    checkScriptBuffer();
-                    scriptCode[ scriptCodeSize ] = 0;
-                    scriptCode[ scriptCodeSize + 1 ] = 0;
+                    checkRawContentBuffer();
+                    rawContent[ rawContentSize ] = 0;
+                    rawContent[ rawContentSize + 1 ] = 0;
                     currToken.tid = ID_COMMENT;
-                    processListing(TokenizerString(scriptCode, scriptCodeSize - 3));
+                    int size = scriptEnd ? rawContentSize - 3 : rawContentSize - 1;
+                    processListing(TokenizerString(rawContent, size));
                     processToken();
                     currToken.tid = ID_COMMENT + ID_CLOSE_TAG;
                     processToken();
-                    scriptCodeSize = 0;
+                    rawContentSize = 0;
                 }
                 comment = false;
                 return; // Finished parsing comment
@@ -619,7 +636,7 @@ void HTMLTokenizer::parseDoctype(TokenizerString &src)
                 isWhitespace = c == '\r' || c == '\n' || c == '\t' || c == ' ';
             }
         }
-        
+
         switch (doctypeToken.state) {
             case DoctypeBegin: {
                 doctypeToken.state = DoctypeBeforeName;
@@ -820,14 +837,14 @@ void HTMLTokenizer::parseDoctype(TokenizerString &src)
 
 void HTMLTokenizer::parseServer(TokenizerString &src)
 {
-    checkScriptBuffer(src.length());
+    checkRawContentBuffer(src.length());
     while ( !src.isEmpty() ) {
-        scriptCode[ scriptCodeSize++ ] = *src;
+        rawContent[ rawContentSize++ ] = *src;
         if (src->unicode() == '>' &&
-            scriptCodeSize > 1 && scriptCode[scriptCodeSize-2] == '%') {
+            rawContentSize > 1 && rawContent[rawContentSize-2] == '%') {
             ++src;
             server = false;
-            scriptCodeSize = 0;
+            rawContentSize = 0;
             return; // Finished parsing server include
         }
         ++src;
@@ -1034,6 +1051,7 @@ void HTMLTokenizer::parseEntity(TokenizerString &src, QChar *&dest, bool start)
                     src.prepend( TokenizerString(QString::fromAscii(cBuffer+entityLen, rem)) );
                 }
                 src.push( EntityChar );
+                rawContentSinceLastEntity = -1;
             } else {
 #ifdef TOKEN_DEBUG
                 kDebug( 6036 ) << "unknown entity!";
@@ -1044,6 +1062,7 @@ void HTMLTokenizer::parseEntity(TokenizerString &src, QChar *&dest, bool start)
                 for(unsigned int i = 0; i < cBufferPos; i++)
                     dest[i] = cBuffer[i];
                 dest += cBufferPos;
+                rawContentSinceLastEntity += cBufferPos+1;
                 if (pre)
                     prePos += cBufferPos+1;
             }
@@ -1058,7 +1077,7 @@ void HTMLTokenizer::parseEntity(TokenizerString &src, QChar *&dest, bool start)
 void HTMLTokenizer::parseTag(TokenizerString &src)
 {
     assert(!Entity );
-    checkScriptBuffer( src.length() );
+    checkRawContentBuffer( src.length() );
 
     while ( !src.isEmpty() )
     {
@@ -1087,7 +1106,7 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                         doctypeSearchCount++; // A '!' is also part of doctype, so we are moving through that still as well
                     else
                         doctypeSearchCount = 0;
-                    
+
                     if (searchCount == 4)
                     {
 #ifdef TOKEN_DEBUG
@@ -1110,7 +1129,7 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                 else
                     searchCount = 0; // Stop looking for '<!--' sequence
             }
-            
+
             if (doctypeSearchCount > 0) {
                 if((*src).toLower() == doctypeStart[doctypeSearchCount]) {
                     doctypeSearchCount++;
@@ -1123,7 +1142,7 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                         doctypeComment = NoDoctypeComment;
                         doctypeToken.reset();
                         doctype = true;
-                        
+
                         parseDoctype(src);
                         return;
                     }
@@ -1214,7 +1233,7 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                     {
                         tag = SearchValue;
                         *dest++ = 0;
-                        attrName.clear();
+                        attrName = DOMString("");
                     }
                     else
                         tag = AttributeName;
@@ -1257,7 +1276,7 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                                 cBuffer[cBufferPos - 1] = '/';
                             }
                             if (!a)
-                                attrName = QLatin1String(QByteArray(cBuffer, cBufferPos+1).data());
+                                attrName = DOMString(cBuffer, cBufferPos);
                         }
 
                         dest = buffer;
@@ -1279,7 +1298,7 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
             }
             if ( cBufferPos == CBUFLEN ) {
                 cBuffer[cBufferPos] = '\0';
-                attrName = QLatin1String(QByteArray(cBuffer, cBufferPos+1).data());
+                attrName = DOMString(cBuffer, cBufferPos);
                 dest = buffer;
                 *dest++ = 0;
                 tag = SearchEqual;
@@ -1307,7 +1326,7 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                     {
                         tag = SearchValue;
                         *dest++ = 0;
-                        attrName.clear();
+                        attrName = DOMString("");
                     }
                     else {
                         DOMString v("");
@@ -1399,7 +1418,13 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                     }
                     // no quotes. Every space means end of value
                     // '/' does not delimit in IE!
-                    if ( curchar <= ' ' || curchar == '>' )
+                    // HTML5: must not contain any literal space characters, any U+0022 QUOTATION MARK (") characters,
+                    // U+0027 APOSTROPHE (') characters, U+003D EQUALS SIGN (=) characters, U+003C LESS-THAN SIGN (<) characters,
+                    // U+003E GREATER-THAN SIGN (>) characters, or U+0060 GRAVE ACCENT (`) characters, and must not be the empty string.
+                    // Real life: images.google.com uses URLs including form arguments (foo=bar)
+                    // in unquoted parameters --- with an html5 <!doctype html> DTD.
+                    // Real life takes priority, so we accept at least =
+                    if ( curchar <= ' ' || curchar == '>' || curchar == '\'' || curchar == '"' || curchar == '<' || /*curchar == '=' ||*/ curchar == '`' )
                     {
                         DOMString v(buffer+1, dest-buffer-1);
                         currToken.addAttribute(parser->docPtr(), buffer, attrName, v);
@@ -1443,21 +1468,22 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
 #if defined(TOKEN_DEBUG) && TOKEN_DEBUG > 0
             kDebug( 6036 ) << "appending Tag: " << tagID;
 #endif
-            // If the tag requires an end tag it cannot be flat,
-            // unless we are using the HTML parser to parse XHTML
-            // The only exception is SCRIPT and priority 0 tokens.
+            // When parsing HTML flat tags like <div /> should 
+	    // be ignored, the only exception is SCRIPT, and 
+	    // tags with forbidden end-tags
             if (tagID < ID_CLOSE_TAG && tagID != ID_SCRIPT &&
-                DOM::endTagRequirement(tagID) == DOM::REQUIRED &&
+                DOM::endTagRequirement(tagID) != DOM::FORBIDDEN &&
                 parser->doc()->htmlMode() != DocumentImpl::XHtml)
                 currToken.flat = false;
 
             bool beginTag = !currToken.flat && (tagID < ID_CLOSE_TAG);
+            HTMLScriptElementImpl* prevScriptElem = 0;
 
             if(tagID >= ID_CLOSE_TAG)
                 tagID -= ID_CLOSE_TAG;
-            else if ( !brokenScript && tagID == ID_SCRIPT ) {
+            else if ( tagID == ID_SCRIPT ) {
+                prevScriptElem = parser->currentScriptElement();
                 DOMStringImpl* a = 0;
-                bool foundTypeAttribute = false;
                 scriptSrc.clear(); scriptSrcCharset.clear();
                 if ( currToken.attrs && /* potentially have a ATTR_SRC ? */
                      view &&  /* are we a regular tokenizer or just for innerHTML ? */
@@ -1469,65 +1495,17 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                         scriptSrcCharset = DOMString(a).string().trimmed();
                     if ( scriptSrcCharset.isEmpty() && view)
                         scriptSrcCharset = parser->doc()->view()->part()->encoding();
-                    /* Check type before language, since language is deprecated */
-                    if ((a = currToken.attrs->getValue(ATTR_TYPE)) != 0 && !DOMString(a).string().isEmpty())
-                        foundTypeAttribute = true;
-                    else
-                        a = currToken.attrs->getValue(ATTR_LANGUAGE);
                 }
                 javascript = true;
-
-                if( foundTypeAttribute ) {
-                    /*
-                        Mozilla 1.5 doesn't accept the text/javascript1.x formats, but WinIE 6 does.
-                        Mozilla 1.5 doesn't accept text/jscript, text/ecmascript, and text/livescript, but WinIE 6 does.
-			Mozilla 1.5 accepts application/x-javascript, WinIE 6 doesn't.
-                        Mozilla 1.5 allows leading and trailing whitespace, but WinIE 6 doesn't.
-                        Mozilla 1.5 and WinIE 6 both accept the empty string, but neither accept a whitespace-only string.
-                        We want to accept all the values that either of these browsers accept, but not other values.
-                     */
-                    QString type = DOMString(a).string().trimmed().toLower();
-                    if( type.compare("text/javascript") != 0 &&
-                        type.compare("text/javascript1.0") != 0 &&
-                        type.compare("text/javascript1.1") != 0 &&
-                        type.compare("text/javascript1.2") != 0 &&
-                        type.compare("text/javascript1.3") != 0 &&
-                        type.compare("text/javascript1.4") != 0 &&
-                        type.compare("text/javascript1.5") != 0 &&
-                        type.compare("text/jscript") != 0 &&
-                        type.compare("text/ecmascript") != 0 &&
-                        type.compare("text/livescript") != 0 &&
-			type.compare("application/x-javascript") != 0 &&
-			type.compare("application/x-ecmascript") != 0 &&
-			type.compare("application/javascript") != 0 &&
-			type.compare("application/ecmascript") != 0 )
-                        javascript = false;
-                } else if( a ) {
-                    /*
-                     Mozilla 1.5 doesn't accept jscript or ecmascript, but WinIE 6 does.
-                     Mozilla 1.5 accepts javascript1.0, javascript1.4, and javascript1.5, but WinIE 6 accepts only 1.1 - 1.3.
-                     Neither Mozilla 1.5 nor WinIE 6 accept leading or trailing whitespace.
-                     We want to accept all the values that either of these browsers accept, but not other values.
-                     */
-                    QString lang = DOMString(a).string();
-                    lang = lang.toLower();
-                    if( lang.compare("") != 0 &&
-                        lang.compare("javascript") != 0 &&
-                        lang.compare("javascript1.0") != 0 &&
-                        lang.compare("javascript1.1") != 0 &&
-                        lang.compare("javascript1.2") != 0 &&
-                        lang.compare("javascript1.3") != 0 &&
-                        lang.compare("javascript1.4") != 0 &&
-                        lang.compare("javascript1.5") != 0 &&
-                        lang.compare("ecmascript") != 0 &&
-                        lang.compare("livescript") != 0 &&
-                        lang.compare("jscript") )
-                        javascript = false;
-                }
             }
 
             processToken();
 
+            if (javascript) {
+                HTMLScriptElementImpl* sc = parser->currentScriptElement();
+                javascript = (sc && sc != prevScriptElem) ? sc->isValidScript() : false;
+            }
+
             if ( parser->selectMode() && beginTag)
                 discard = AllDiscard;
 
@@ -1547,17 +1525,19 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                     searchStopper = scriptEnd;
                     searchStopperLen = 8;
                     script = true;
-                    parseSpecial(src);
+                    parseRawContent(src);
                 }
-                else if (tagID < ID_CLOSE_TAG) // Handle <script src="foo"/>
+                else if (tagID < ID_CLOSE_TAG) { // Handle <script src="foo"/>
+                    script = true;
                     scriptHandler();
+                }
                 break;
             case ID_STYLE:
                 if (beginTag) {
                     searchStopper = styleEnd;
                     searchStopperLen = 7;
                     style = true;
-                    parseSpecial(src);
+                    parseRawContent(src);
                 }
                 break;
             case ID_TEXTAREA:
@@ -1566,7 +1546,8 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                     searchStopperLen = 10;
                     textarea = true;
                     discard = NoneDiscard;
-                    parseSpecial(src);
+                    rawContentSinceLastEntity = 0;
+                    parseRawContent(src);
                 }
                 break;
             case ID_TITLE:
@@ -1574,7 +1555,8 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                     searchStopper = titleEnd;
                     searchStopperLen = 7;
                     title = true;
-                    parseSpecial(src);
+                    rawContentSinceLastEntity = 0;
+                    parseRawContent(src);
                 }
                 break;
             case ID_XMP:
@@ -1582,7 +1564,7 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                     searchStopper = xmpEnd;
                     searchStopperLen = 5;
                     xmp = true;
-                    parseSpecial(src);
+                    parseRawContent(src);
                 }
                 break;
             case ID_SELECT:
@@ -1647,12 +1629,13 @@ inline bool HTMLTokenizer::continueProcessing(int& processedCount)
     return true;
 }
 
+
+#include "khtmlpart_p.h"
 void HTMLTokenizer::write( const TokenizerString &str, bool appendData )
 {
 #ifdef TOKEN_DEBUG
     kDebug( 6036 ) << this << " Tokenizer::write(\"" << str.toString() << "\"," << appendData << ")";
 #endif
-
     if ( !buffer )
         return;
 
@@ -1714,15 +1697,15 @@ void HTMLTokenizer::write( const TokenizerString &str, bool appendData )
         else if ( plaintext )
             parseText( src );
         else if (script)
-            parseSpecial(src);
+            parseRawContent(src);
         else if (style)
-            parseSpecial(src);
+            parseRawContent(src);
         else if (xmp)
-            parseSpecial(src);
+            parseRawContent(src);
         else if (textarea)
-            parseSpecial(src);
+            parseRawContent(src);
         else if (title)
-            parseSpecial(src);
+            parseRawContent(src);
         else if (comment)
             parseComment(src);
         else if (doctypeComment && doctypeComment != DoctypeCommentEnd && doctypeComment != DoctypeCommentBogus)
@@ -1758,8 +1741,6 @@ void HTMLTokenizer::write( const TokenizerString &str, bool appendData )
                 tquote = NoQuote;
                 parseProcessingInstruction(src);
                 continue;
-
-                break;
             }
             case '%':
                 if (!brokenServer) {
@@ -1794,10 +1775,16 @@ void HTMLTokenizer::write( const TokenizerString &str, bool appendData )
             // ### Gecko and MSIE though only ignores LF immediately after
             // starttags and only for PRE elements -- asj (28/06-2005)
             if ( pending )
+            {
                 if (!select)
+                {
                     addPending();
+                }
                 else
+                {
                     pending = NonePending;
+                }
+            }
 
             // Cancel unused discards
             discard = NoneDiscard;
@@ -1910,81 +1897,76 @@ void HTMLTokenizer::timerEvent( QTimerEvent *e )
     if ( e->timerId() == m_yieldTimer ) {
         killTimer(m_yieldTimer);
         m_yieldTimer = 0;
-        write( TokenizerString(), true ); 
-    } else if ( e->timerId() == m_autoCloseTimer && cachedScript.isEmpty() ) {
-         finish();
+        write( TokenizerString(), true );
+    } else if ( e->timerId() == m_externalScriptsTimerId ) {
+        if (view && view->hasLayoutPending()) {
+            // all stylesheets are loaded but the style modifications 
+            // they triggered have yet to be applied, BBIAB
+            return;
+        }
+        killTimer(m_externalScriptsTimerId);
+        m_externalScriptsTimerId = 0;
+        notifyFinished(0);
     }
 }
 
-void HTMLTokenizer::setAutoClose( bool b ) {
-    killTimer( m_autoCloseTimer );
-    m_autoCloseTimer = 0;
-    if ( b )
-        m_autoCloseTimer = startTimer(100);
-}
-
 void HTMLTokenizer::end()
 {
-    if ( buffer == 0 ) {
-        emit finishedParsing();
-        return;
+    if ( buffer ) {
+        // parseTag is using the buffer for different matters
+        if ( !tag )
+            processToken();
+
+        if(buffer)
+            KHTML_DELETE_QCHAR_VEC(buffer);
+
+        if(rawContent)
+            KHTML_DELETE_QCHAR_VEC(rawContent);
+
+        rawContent = 0;
+        rawContentSize = rawContentMaxSize = rawContentResync = 0;
+        buffer = 0;
     }
-
-    // parseTag is using the buffer for different matters
-    if ( !tag )
-        processToken();
-
-    if(buffer)
-        KHTML_DELETE_QCHAR_VEC(buffer);
-
-    if(scriptCode)
-        KHTML_DELETE_QCHAR_VEC(scriptCode);
-
-    scriptCode = 0;
-    scriptCodeSize = scriptCodeMaxSize = scriptCodeResync = 0;
-    buffer = 0;
     emit finishedParsing();
 }
 
 void HTMLTokenizer::finish()
 {
-    if ( m_autoCloseTimer ) {
-        killTimer( m_autoCloseTimer );
-        m_autoCloseTimer = 0;
-    }
-    // do this as long as we don't find matching comment ends
-    while((title || script || comment || server) && scriptCode && scriptCodeSize)
+    // The purpose of this iteration is to recover from 'raw content' tokenizing mode.
+    // In this mode, any error such as the lack of a closing tag (for the considered element) or of a closing comment,
+    // would result in the entire document being absorbed in one node.
+    // When it happens, we simply put back in the input buffer what this mode's output has accumulated so far,
+    // and retokenize after either disabling the 'raw content' mode (by setting the corresponding members to false)
+    // or after setting a few flags disabling some lax parsing 'features' (brokenComments/brokenServer).
+    while((title || comment || server) && rawContent && rawContentSize)
     {
         // we've found an unmatched comment start
         if (comment)
             brokenComments = true;
         else if (server)
             brokenServer = true;
-        else if (script)
-            brokenScript = true;
 
-        checkScriptBuffer();
-        scriptCode[ scriptCodeSize ] = 0;
-        scriptCode[ scriptCodeSize + 1 ] = 0;
+        checkRawContentBuffer();
+        rawContent[ rawContentSize ] = 0;
+        rawContent[ rawContentSize + 1 ] = 0;
         int pos;
         QString food;
-        if (title || style || script)
-            food.setUnicode(scriptCode, scriptCodeSize);
-        else if (server) {
+        if (title || style || script || textarea) {
+            rawContentSinceLastEntity = 0;
+            food.setUnicode(rawContent, rawContentSize);
+        } else if (server) {
             food = "<";
-            food += QString(scriptCode, scriptCodeSize);
+            food += QString(rawContent, rawContentSize);
         }
         else {
-            pos = QString::fromRawData(scriptCode, scriptCodeSize).indexOf('>');
-            food.setUnicode(scriptCode+pos+1, scriptCodeSize-pos-1); // deep copy
+            pos = QString::fromRawData(rawContent, rawContentSize).indexOf('>');
+            food.setUnicode(rawContent+pos+1, rawContentSize-pos-1); // deep copy
         }
-        KHTML_DELETE_QCHAR_VEC(scriptCode);
-        scriptCode = 0;
-        scriptCodeSize = scriptCodeMaxSize = scriptCodeResync = 0;
-        if (script)
-            scriptHandler();
+        KHTML_DELETE_QCHAR_VEC(rawContent);
+        rawContent = 0;
+        rawContentSize = rawContentMaxSize = rawContentResync = 0;
 
-        comment = title = server = script = false;
+        comment = server = title = false;
         if ( !food.isEmpty() )
             write(food, true);
     }
@@ -2024,27 +2006,28 @@ void HTMLTokenizer::processToken()
     dest = buffer;
 
 #ifdef TOKEN_DEBUG
-    QString name = QString( getTagName(currToken.tid) );
     QString text;
+    bool closing = (currToken.tid > ID_CLOSE_TAG);
+    int rid = currToken.tid-(closing ? ID_CLOSE_TAG : 0);
     if(currToken.text)
         text = QString::fromRawData(currToken.text->s, currToken.text->l);
-
-    kDebug( 6036 ) << "Token --> " << name << "   id = " << currToken.tid;
+    kDebug( 6036 ) << "Token -->" << LocalName::fromId(localNamePart(rid)).toString()
+                   << "id =" << currToken.tid << "closing ="<< closing;
     if (currToken.flat)
         kDebug( 6036 ) << "Token is FLAT!";
     if(!text.isNull())
         kDebug( 6036 ) << "text: \"" << text << "\"";
     unsigned long l = currToken.attrs ? currToken.attrs->length() : 0;
+ 
     if(l) {
         kDebug( 6036 ) << "Attributes: " << l;
         for (unsigned long i = 0; i < l; ++i) {
             NodeImpl::Id tid = currToken.attrs->idAt(i);
             DOMString value = currToken.attrs->valueAt(i);
-            kDebug( 6036 ) << "    " << tid << " " << parser->doc()->document()->getName(NodeImpl::AttributeId, tid).string()
+            kDebug( 6036 ) << "    " << tid << " " << LocalName::fromId(localNamePart(tid)).toString()
                             << "=\"" << value.string() << "\"" << endl;
         }
     }
-    kDebug( 6036 );
 #endif
 
     // In some cases, parseToken() can cause javascript code to be executed
@@ -2092,20 +2075,31 @@ void HTMLTokenizer::enlargeBuffer(int len)
     size = newsize;
 }
 
-void HTMLTokenizer::enlargeScriptBuffer(int len)
+void HTMLTokenizer::enlargeRawContentBuffer(int len)
 {
-    int newsize = qMax(scriptCodeMaxSize*2, scriptCodeMaxSize+len);
-    scriptCode = KHTML_REALLOC_QCHAR_VEC(scriptCode, newsize);
-    scriptCodeMaxSize = newsize;
+    int newsize = qMax(rawContentMaxSize*2, rawContentMaxSize+len);
+    rawContent = KHTML_REALLOC_QCHAR_VEC(rawContent, newsize);
+    rawContentMaxSize = newsize;
 }
 
-void HTMLTokenizer::notifyFinished(CachedObject* /*finishedObj*/)
+void HTMLTokenizer::notifyFinished(CachedObject* finishedObj)
 {
     assert(!cachedScript.isEmpty());
-    bool done = false;
-    while (!done && cachedScript.head()->isLoaded()) {
+    // Make external scripts wait for external stylesheets.
+    // FIXME: This needs to be done for inline scripts too.
+    m_hasScriptsWaitingForStylesheets = !parser->doc()->haveStylesheetsLoaded();
+    if (m_hasScriptsWaitingForStylesheets) {
+        kDebug( 6036 ) << "Delaying script execution until stylesheets have loaded.";
+        return;
+    }
+    kDebug( 6036 ) << (finishedObj ? "Processing an external script"  : 
+                                     "Continuing processing of delayed external scripts");
 
-        kDebug( 6036 ) << "Finished loading an external script";
+    bool done = false;
+    m_scriptTime.start();
+    while (!done && cachedScript.head()->isLoaded()) {
+        if (!continueProcessingScripts())
+            break;
 
         CachedScript* cs = cachedScript.dequeue();
         DOMString scriptSource = cs->script();
@@ -2122,10 +2116,16 @@ void HTMLTokenizer::notifyFinished(CachedObject* /*finishedObj*/)
 	scriptExecution( scriptSource.string(), cachedScriptUrl );
 
         done = cachedScript.isEmpty();
-
+        if (done) {
+            assert(!m_hasScriptsWaitingForStylesheets);
+        } else if (m_hasScriptsWaitingForStylesheets) {
+            // flag has changed during the script execution,
+            // so we need to wait for stylesheets again.
+             done = true;
+        }
         // 'script' is true when we are called synchronously from
         // scriptHandler(). In that case scriptHandler() will take care
-        // of 'scriptOutput'.
+        // of the pending queue.
         if ( !script ) {
             while (pendingQueue.count() > 1) {
                TokenizerString t = pendingQueue.pop();
@@ -2140,6 +2140,24 @@ void HTMLTokenizer::notifyFinished(CachedObject* /*finishedObj*/)
     }
 }
 
+bool HTMLTokenizer::continueProcessingScripts()
+{
+    if (m_externalScriptsTimerId)
+        return false;
+    if (m_scriptTime.elapsed() > m_tokenizerYieldDelay && m_documentTokenizer) {
+        if ( (m_externalScriptsTimerId = startTimer(0)) )
+            return false;
+    }
+    return true;
+}
+
+void HTMLTokenizer::executeScriptsWaitingForStylesheets()
+{
+    assert(parser->doc()->haveStylesheetsLoaded());
+    if (m_hasScriptsWaitingForStylesheets)
+        notifyFinished(0);
+}
+
 bool HTMLTokenizer::isWaitingForScripts() const
 {
     return cachedScript.count();
