@@ -137,9 +137,8 @@ using namespace DOM;
                 const QComboBox* cb = qobject_cast<const QComboBox*>(widget);
                 const QStyleOptionComboBox* cbOpt = qstyleoption_cast<const QStyleOptionComboBox*>(opt);
 
-                QFontMetrics fm = cb->fontMetrics();
-
                 if (cb && cbOpt) {
+                    QFontMetrics fm = cb->fontMetrics();
                     // Compute content width; Qt uses the usual +4 magic number for icon/text margin
                     int maxW = 0;
                     for (int c = 0; c < cb->count(); ++c) {
@@ -331,7 +330,10 @@ RenderButton::RenderButton(HTMLGenericFormElementImpl *element)
 
 short RenderButton::baselinePosition( bool f ) const
 {
-    return RenderWidget::baselinePosition( f ) - 2;
+    int ret = (height()-RenderWidget::paddingTop()-RenderWidget::paddingBottom()+1)/2;
+    ret += marginTop() + RenderWidget::paddingTop();
+    ret += ((fontMetrics( f ).ascent())/2)-1;
+    return ret;
 }
 
 void RenderButton::layout()
@@ -704,7 +706,7 @@ LineEditWidget::LineEditWidget(DOM::HTMLInputElementImpl* input, KHTMLView* view
     m_kwp->setIsRedirected( true );
     setMouseTracking(true);
     KActionCollection *ac = new KActionCollection(this);
-    m_spellAction = KStandardAction::spelling( this, SLOT( slotCheckSpelling() ), ac );
+    m_spellAction = KStandardAction::spelling( this, SLOT(slotCheckSpelling()), ac );
 
     setCompletionBox( new CompletionWidget( this ) );
     completionBox()->setObjectName("completion box");
@@ -721,9 +723,9 @@ void LineEditWidget::slotCheckSpelling()
         return;
     }
     Sonnet::Dialog *spellDialog = new Sonnet::Dialog(new Sonnet::BackgroundChecker(this), 0);
-    connect(spellDialog, SIGNAL(replace( const QString&, int,const QString&)), this, SLOT(spellCheckerCorrected( const QString&, int,const QString&)));
-    connect(spellDialog, SIGNAL(misspelling( const QString&, int)), this, SLOT(spellCheckerMisspelling(const QString &,int)));
-    connect(spellDialog, SIGNAL(done(const QString&)), this, SLOT(slotSpellCheckDone(const QString&)));
+    connect(spellDialog, SIGNAL(replace(QString,int,QString)), this, SLOT(spellCheckerCorrected(QString,int,QString)));
+    connect(spellDialog, SIGNAL(misspelling(QString,int)), this, SLOT(spellCheckerMisspelling(QString,int)));
+    connect(spellDialog, SIGNAL(done(QString)), this, SLOT(slotSpellCheckDone(QString)));
     connect(spellDialog, SIGNAL(cancel()), this, SLOT(spellCheckerFinished()));
     connect(spellDialog, SIGNAL(stop()), this, SLOT(spellCheckerFinished()));
     spellDialog->setBuffer(text());
@@ -967,11 +969,6 @@ void LineEditWidget::clearHistoryActivated()
 
 void LineEditWidget::paintEvent( QPaintEvent *pe )
 {
-    if (!hasFrame()) {
-        QPainter p(this);
-        p.fillRect(pe->rect(), palette().brush(QPalette::Base));
-        p.end();
-    }
     KLineEdit::paintEvent( pe );
 }
 
@@ -1012,11 +1009,11 @@ void LineEditWidget::mouseMoveEvent(QMouseEvent *e)
 // -----------------------------------------------------------------------------
 
 RenderLineEdit::RenderLineEdit(HTMLInputElementImpl *element)
-    : RenderFormElement(element)
+    : RenderFormElement(element), m_blockElementUpdates(false)
 {
     LineEditWidget *edit = new LineEditWidget(element, view(), view()->widget());
     connect(edit,SIGNAL(returnPressed()), this, SLOT(slotReturnPressed()));
-    connect(edit,SIGNAL(textEdited(QString)),this,SLOT(slotTextEdited(QString)));
+    connect(edit,SIGNAL(textChanged(QString)),this,SLOT(slotTextChanged(QString)));
 
     if(element->inputType() == HTMLInputElementImpl::PASSWORD)
         edit->setEchoMode( QLineEdit::Password );
@@ -1049,20 +1046,21 @@ void RenderLineEdit::setStyle(RenderStyle* _style)
     RenderFormElement::setStyle( _style );
 
     widget()->setAlignment(textAlignment());
+
     bool showClearButton = (!shouldDisableNativeBorders() && !_style->hasBackgroundImage());
-    static_cast<LineEditWidget*>(widget())->setClearButtonShown( showClearButton );
-    if (showClearButton) {
+
+    if (!showClearButton && widget()->isClearButtonShown()) {
+        widget()->setClearButtonShown(false);
+    }
+    else if (showClearButton && !widget()->isClearButtonShown()) {
+        widget()->setClearButtonShown(true);
         QObjectList children = widget()->children();
         foreach (QObject* object, children) {
             QWidget *w = qobject_cast<QWidget*>(object);
-            if (w && !w->isWindow()) {
+            if (w && !w->isWindow() && (w->objectName() == "KLineEditButton")) {
                 // this duplicates KHTMLView's handleWidget but this widget
                 // is created on demand, so it might not be here at ChildPolished time
-                w->setObjectName("KHTMLLineEditButton");
                 w->installEventFilter(view());
-                w->setAttribute(Qt::WA_NoSystemBackground);
-                w->setAttribute(Qt::WA_WState_InPaintEvent);
-                w->setAttribute(Qt::WA_OpaquePaintEvent);
             }
         }
     }
@@ -1142,19 +1140,25 @@ void RenderLineEdit::updateFromElement()
     }
 
     if (element()->value().string() != widget()->text()) {
+        m_blockElementUpdates = true;  // Do not block signals here (#188374)
         int pos = widget()->cursorPosition();
         widget()->setText(element()->value().string());
         widget()->setCursorPosition(pos);
+        m_blockElementUpdates = false;
     }
     widget()->setReadOnly(element()->readOnly());
 
+    widget()->setClickMessage(element()->placeholder().string().remove(QLatin1Char('\n')).remove(QLatin1Char('\r')));
+
     RenderFormElement::updateFromElement();
 }
 
-void RenderLineEdit::slotTextEdited(const QString &string)
+void RenderLineEdit::slotTextChanged(const QString &string)
 {
+    if (m_blockElementUpdates) return;
+
     // don't use setValue here!
-    element()->m_value = string;
+    element()->m_value = string.isNull() ? DOMString("") : string;
     element()->m_unsubmittedFormChange = true;
 }
 
@@ -1399,8 +1403,8 @@ RenderFileButton::RenderFileButton(HTMLInputElementImpl *element)
     w->completionObject()->setDir(KGlobalSettings::documentPath());
 
     connect(w->lineEdit(), SIGNAL(returnPressed()), this, SLOT(slotReturnPressed()));
-    connect(w->lineEdit(), SIGNAL(textChanged(const QString &)),this,SLOT(slotTextChanged(const QString &)));
-    connect(w, SIGNAL(urlSelected(const KUrl &)),this,SLOT(slotUrlSelected(const KUrl &)));
+    connect(w->lineEdit(), SIGNAL(textChanged(QString)),this,SLOT(slotTextChanged(QString)));
+    connect(w, SIGNAL(urlSelected(KUrl)),this,SLOT(slotUrlSelected(KUrl)));
 
     setQWidget(w);
     m_haveFocus = false;
@@ -2000,7 +2004,7 @@ ListBoxWidget* RenderSelect::createListBox()
 {
     ListBoxWidget *lb = new ListBoxWidget(view()->widget());
     lb->setSelectionMode(m_multiple ? QListWidget::ExtendedSelection : QListWidget::SingleSelection);
-    connect( lb, SIGNAL( itemSelectionChanged() ), this, SLOT( slotSelectionChanged() ) );
+    connect( lb, SIGNAL(itemSelectionChanged()), this, SLOT(slotSelectionChanged()) );
     m_ignoreSelectEvents = false;
     lb->setMouseTracking(true);
 
@@ -2297,6 +2301,7 @@ void RenderTextArea::updateFromElement()
 {
     TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
     w->setReadOnly(element()->readOnly());
+    w->setClickMessage(element()->placeholder().string());
     RenderFormElement::updateFromElement();
 }
 
