@@ -23,7 +23,7 @@
 #define KIO_JOBCLASSES_H
 
 #include <QtCore/QObject>
-#include <QtCore/QLinkedList>
+#include <QtCore/QLinkedList> // KDE5: remove
 #include <QtCore/QStringList>
 
 #include <kurl.h>
@@ -32,7 +32,6 @@
 
 #include <kcompositejob.h>
 
-
 namespace KIO {
 
     /**
@@ -175,7 +174,9 @@ namespace KIO {
          * @deprecated you should use job->ui()->setWindow(parent)
          *             and job->ui()->showErrorMessage() instead
          */
+#ifndef KDE_NO_DEPRECATED
         KDE_DEPRECATED void showErrorDialog( QWidget *parent = 0 );
+#endif
 
         /**
          * Returns whether the user should be asked about things when the job
@@ -362,7 +363,7 @@ namespace KIO {
         static void removeOnHold();
 
         /**
-         * Returns true if redirections are handled internally, the default.
+         * Returns true when redirections are handled internally, the default.
          *
          * @since 4.4
          */
@@ -464,7 +465,9 @@ namespace KIO {
          * (e.g. with FTP).
          * @param source true for "source" mode, false for "dest" mode
          */
+#ifndef KDE_NO_DEPRECATED
         KDE_DEPRECATED void setSide( bool source );
+#endif
 
         /**
          * Selects the level of @p details we want.
@@ -593,7 +596,9 @@ namespace KIO {
          * @deprecated not needed, this is false for KIO::get and true for KIO::put,
          *             automatically since KDE-4.2.1
          */
+#ifndef KDE_NO_DEPRECATED
         KDE_DEPRECATED void setReportDataSent(bool enabled);
+#endif
 
         /**
          *  Returns whether the job reports the amount of data that has been
@@ -602,7 +607,9 @@ namespace KIO {
          * @deprecated not needed, this is false for KIO::get and true for KIO::put,
          *             automatically since KDE-4.2.1 (and not useful as public API)
          */
+#ifndef KDE_NO_DEPRECATED
         KDE_DEPRECATED bool reportDataSent() const;
+#endif
 
         /**
          * Call this in the slot connected to result,
@@ -706,6 +713,7 @@ namespace KIO {
         Q_PRIVATE_SLOT(d_func(), void slotPostRedirection())
         Q_PRIVATE_SLOT(d_func(), void slotNeedSubUrlData())
         Q_PRIVATE_SLOT(d_func(), void slotSubUrlData(KIO::Job*, const QByteArray &))
+        Q_PRIVATE_SLOT(d_func(), void slotDataReqFromDevice())
         Q_DECLARE_PRIVATE(TransferJob)
 
         // A FileCopyJob may control one or more TransferJobs
