@@ -29,6 +29,8 @@
 
 #include <config.h>
 #include <config-acl.h>
+#include <config-kioslave-file.h>
+
 
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -314,7 +316,7 @@ void FileProtocol::get( const KUrl& url )
         return;
     }
 
-#ifdef HAVE_FADVISE
+#if HAVE_FADVISE
     posix_fadvise( fd, 0, 0, POSIX_FADV_SEQUENTIAL);
 #endif
 
@@ -734,10 +736,14 @@ void FileProtocol::put( const KUrl& url, int _mode, KIO::JobFlags _flags )
         if ( dt.isValid() ) {
             KDE_struct_stat dest_statbuf;
             if (KDE::stat( dest_orig, &dest_statbuf ) == 0) {
-                struct utimbuf utbuf;
-                utbuf.actime = dest_statbuf.st_atime; // access time, unchanged
-                utbuf.modtime = dt.toTime_t(); // modification time
-                KDE::utime( dest_orig, &utbuf );
+                struct timeval utbuf[2];
+                // access time
+                utbuf[0].tv_sec = dest_statbuf.st_atime; // access time, unchanged  ## TODO preserve msec
+                utbuf[0].tv_usec = 0;
+                // modification time
+                utbuf[1].tv_sec = dt.toTime_t();
+                utbuf[1].tv_usec = dt.time().msec() * 1000;
+                utimes( QFile::encodeName(dest_orig), utbuf );
             }
         }
 
@@ -790,10 +796,15 @@ bool FileProtocol::createUDSEntry( const QString & filename, const QByteArray &
 
     if ( KDE_lstat( path.data(), &buff ) == 0 )  {
 
+        if (details > 2) {
+            entry.insert( KIO::UDSEntry::UDS_DEVICE_ID, buff.st_dev );
+            entry.insert( KIO::UDSEntry::UDS_INODE, buff.st_ino );
+        }
+
         if (S_ISLNK(buff.st_mode)) {
 
             char buffer2[ 1000 ];
-            int n = readlink( path.data(), buffer2, 1000 );
+            int n = readlink( path.data(), buffer2, 999 );
             if ( n != -1 ) {
                 buffer2[ n ] = 0;
             }
@@ -896,6 +907,7 @@ void FileProtocol::mount( bool _ro, const char *_fstype, const QString& _dev, co
 {
     kDebug(7101) << "fstype=" << _fstype;
 
+#ifndef _WIN32_WCE
 #ifdef HAVE_VOLMGT
 	/*
 	 *  support for Solaris volume management
@@ -1030,11 +1042,18 @@ void FileProtocol::mount( bool _ro, const char *_fstype, const QString& _dev, co
         }
     }
 #endif /* ! HAVE_VOLMGT */
+#else
+    QString err;
+    err = i18n("mounting is not supported by wince.");
+    error( KIO::ERR_COULD_NOT_MOUNT, err );
+#endif
+
 }
 
 
 void FileProtocol::unmount( const QString& _point )
 {
+#ifndef _WIN32_WCE
     QByteArray buffer;
 
     KTemporaryFile tmpFile;
@@ -1145,6 +1164,11 @@ void FileProtocol::unmount( const QString& _point )
         finished();
     else
         error( KIO::ERR_COULD_NOT_UNMOUNT, err );
+#else
+    QString err;
+    err = i18n("unmounting is not supported by wince.");
+    error( KIO::ERR_COULD_NOT_MOUNT, err );
+#endif
 }
 
 /*************************************
@@ -1155,6 +1179,7 @@ void FileProtocol::unmount( const QString& _point )
 
 bool FileProtocol::pmount(const QString &dev)
 {
+#ifndef _WIN32_WCE
     QString epath = QString::fromLocal8Bit(qgetenv("PATH"));
     QString path = QLatin1String("/sbin:/bin");
     if (!epath.isEmpty())
@@ -1170,10 +1195,14 @@ bool FileProtocol::pmount(const QString &dev)
     int res = system( buffer.constData() );
 
     return res==0;
+#else
+    return false;
+#endif
 }
 
 bool FileProtocol::pumount(const QString &point)
 {
+#ifndef _WIN32_WCE
     KMountPoint::Ptr mp = KMountPoint::currentMountPoints(KMountPoint::NeedRealDeviceName).findByPath(point);
     if (!mp)
         return false;
@@ -1196,6 +1225,9 @@ bool FileProtocol::pumount(const QString &point)
     int res = system( buffer.data() );
 
     return res==0;
+#else
+    return false;
+#endif
 }
 
 /*************************************
@@ -1294,6 +1326,8 @@ static void appendACLAtoms( const QByteArray & path, UDSEntry& entry, mode_t typ
 }
 #endif
 
+// We could port this to KTempDir::removeDir but then we wouldn't be able to tell the user
+// where exactly the deletion failed, in case of errors.
 bool FileProtocol::deleteRecursive(const QString& path)
 {
     //kDebug() << path;
